@@ -41,6 +41,7 @@
 #include "syscall.h"
 #include "agent.h"
 #include "buffer-registry.h"
+#include "agent-thread.h"
 
 #include "cmd.h"
 
@@ -856,8 +857,11 @@ static int create_connect_relayd(struct lttng_uri *uri,
 
 		/* Check relayd version */
 		ret = relayd_version_check(rsock);
-		if (ret < 0) {
-			ret = LTTNG_ERR_RELAYD_VERSION_FAIL;
+		if (ret == LTTNG_ERR_RELAYD_VERSION_FAIL) {
+			goto close_sock;
+		} else if (ret < 0) {
+			ERR("Unable to reach lttng-relayd");
+			ret = LTTNG_ERR_RELAYD_CONNECT_FAIL;
 			goto close_sock;
 		}
 		consumer->relay_major_version = rsock->major;
@@ -886,6 +890,8 @@ error:
 
 /*
  * Connect to the relayd using URI and send the socket to the right consumer.
+ *
+ * The consumer socket lock must be held by the caller.
  */
 static int send_consumer_relayd_socket(enum lttng_domain_type domain,
 		unsigned int session_id, struct lttng_uri *relayd_uri,
@@ -899,7 +905,7 @@ static int send_consumer_relayd_socket(enum lttng_domain_type domain,
 	/* Connect to relayd and make version check if uri is the control. */
 	ret = create_connect_relayd(relayd_uri, &rsock, consumer);
 	if (ret != LTTNG_OK) {
-		goto error;
+		goto relayd_comm_error;
 	}
 	assert(rsock);
 
@@ -939,10 +945,6 @@ static int send_consumer_relayd_socket(enum lttng_domain_type domain,
 	 */
 
 close_sock:
-	(void) relayd_close(rsock);
-	free(rsock);
-
-error:
 	if (ret != LTTNG_OK) {
 		/*
 		 * The consumer output for this session should not be used anymore
@@ -951,6 +953,10 @@ error:
 		 */
 		consumer->enabled = 0;
 	}
+	(void) relayd_close(rsock);
+	free(rsock);
+
+relayd_comm_error:
 	return ret;
 }
 
@@ -958,6 +964,8 @@ error:
  * Send both relayd sockets to a specific consumer and domain.  This is a
  * helper function to facilitate sending the information to the consumer for a
  * session.
+ *
+ * The consumer socket lock must be held by the caller.
  */
 static int send_consumer_relayd_sockets(enum lttng_domain_type domain,
 		unsigned int session_id, struct consumer_output *consumer,
@@ -1337,6 +1345,21 @@ int cmd_enable_channel(struct ltt_session *session,
 		attr->attr.switch_timer_interval = 0;
 	}
 
+	/* Check for feature support */
+	switch (domain->type) {
+	case LTTNG_DOMAIN_JUL:
+	case LTTNG_DOMAIN_LOG4J:
+	case LTTNG_DOMAIN_PYTHON:
+		if (!agent_tracing_is_enabled()) {
+			DBG("Attempted to enable a channel in an agent domain but the agent thread is not running");
+			ret = LTTNG_ERR_AGENT_TRACING_DISABLED;
+			goto error;
+		}
+		break;
+	default:
+		break;
+	}
+
 	switch (domain->type) {
 	case LTTNG_DOMAIN_KERNEL:
 	{
@@ -1609,6 +1632,16 @@ int cmd_add_context(struct ltt_session *session, enum lttng_domain_type domain,
 	int ret, chan_kern_created = 0, chan_ust_created = 0;
 	char *app_ctx_provider_name = NULL, *app_ctx_name = NULL;
 
+	/*
+	 * Don't try to add a context if the session has been started at
+	 * some point in time before. The tracer does not allow it and would
+	 * result in a corrupted trace.
+	 */
+	if (session->has_been_started) {
+		ret = LTTNG_ERR_TRACE_ALREADY_STARTED;
+		goto end;
+	}
+
 	if (ctx->ctx == LTTNG_EVENT_CONTEXT_APP_CONTEXT) {
 		app_ctx_provider_name = ctx->u.app_ctx.provider_name;
 		app_ctx_name = ctx->u.app_ctx.ctx_name;
@@ -2060,6 +2093,12 @@ static int _cmd_enable_event(struct ltt_session *session,
 
 		assert(usess);
 
+		if (!agent_tracing_is_enabled()) {
+			DBG("Attempted to enable an event in an agent domain but the agent thread is not running");
+			ret = LTTNG_ERR_AGENT_TRACING_DISABLED;
+			goto error;
+		}
+
 		agt = trace_ust_find_agent(usess, domain->type);
 		if (!agt) {
 			agt = agent_create(domain->type);
@@ -3390,10 +3429,12 @@ static
 int clear_metadata_file(int fd)
 {
 	int ret;
+	off_t lseek_ret;
 
-	ret = lseek(fd, 0, SEEK_SET);
-	if (ret < 0) {
+	lseek_ret = lseek(fd, 0, SEEK_SET);
+	if (lseek_ret < 0) {
 		PERROR("lseek");
+		ret = -1;
 		goto end;
 	}
 
@@ -3606,10 +3647,12 @@ static int set_relayd_for_snapshot(struct consumer_output *consumer,
 	rcu_read_lock();
 	cds_lfht_for_each_entry(snap_output->consumer->socks->ht, &iter.iter,
 			socket, node.node) {
+		pthread_mutex_lock(socket->lock);
 		ret = send_consumer_relayd_sockets(0, session->id,
 				snap_output->consumer, socket,
 				session->name, session->hostname,
 				session->live_timer);
+		pthread_mutex_unlock(socket->lock);
 		if (ret != LTTNG_OK) {
 			rcu_read_unlock();
 			goto error;
