@@ -15,7 +15,7 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-#define _GNU_SOURCE
+#define _LGPL_SOURCE
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -24,6 +24,7 @@
 #include <common/common.h>
 #include <common/defaults.h>
 
+#include "consumer.h"
 #include "trace-kernel.h"
 
 /*
@@ -34,10 +35,15 @@ struct ltt_kernel_channel *trace_kernel_get_channel_by_name(
 {
 	struct ltt_kernel_channel *chan;
 
-	if (session == NULL) {
-		ERR("Undefine session");
-		goto error;
-	}
+	assert(session);
+	assert(name);
+
+	/*
+	 * If we receive an empty string for channel name, it means the
+	 * default channel name is requested.
+	 */
+	if (name[0] == '\0')
+		name = DEFAULT_CHANNEL_NAME;
 
 	DBG("Trying to find channel %s", name);
 
@@ -48,33 +54,82 @@ struct ltt_kernel_channel *trace_kernel_get_channel_by_name(
 		}
 	}
 
-error:
 	return NULL;
 }
 
+/*
+ * Find the event for the given channel.
+ */
+struct ltt_kernel_event *trace_kernel_find_event(
+		char *name, struct ltt_kernel_channel *channel,
+		enum lttng_event_type type,
+		struct lttng_filter_bytecode *filter)
+{
+	struct ltt_kernel_event *ev;
+	int found = 0;
+
+	assert(name);
+	assert(channel);
+
+	cds_list_for_each_entry(ev, &channel->events_list.head, list) {
+		if (type != LTTNG_EVENT_ALL && ev->type != type) {
+			continue;
+		}
+		if (strcmp(name, ev->event->name)) {
+			continue;
+		}
+		if ((ev->filter && !filter) || (!ev->filter && filter)) {
+			continue;
+		}
+		if (ev->filter && filter) {
+			if (ev->filter->len != filter->len ||
+					memcmp(ev->filter->data, filter->data,
+						filter->len) != 0) {
+				continue;
+			}
+		}
+		found = 1;
+		break;
+	}
+	if (found) {
+		DBG("Found event %s for channel %s", name,
+			channel->channel->name);
+		return ev;
+	} else {
+		return NULL;
+	}
+}
+
 /*
  * Find the event name for the given channel.
  */
 struct ltt_kernel_event *trace_kernel_get_event_by_name(
-		char *name, struct ltt_kernel_channel *channel)
+		char *name, struct ltt_kernel_channel *channel,
+		enum lttng_event_type type)
 {
 	struct ltt_kernel_event *ev;
+	int found = 0;
 
-	if (channel == NULL) {
-		ERR("Undefine channel");
-		goto error;
-	}
+	assert(name);
+	assert(channel);
 
 	cds_list_for_each_entry(ev, &channel->events_list.head, list) {
-		if (strcmp(name, ev->event->name) == 0) {
-			DBG("Found event by name %s for channel %s", name,
-					channel->channel->name);
-			return ev;
+		if (type != LTTNG_EVENT_ALL && ev->type != type) {
+			continue;
 		}
+		if (strcmp(name, ev->event->name)) {
+			continue;
+		}
+		found = 1;
+		break;
+	}
+	if (found) {
+		DBG("Found event %s for channel %s", name,
+			channel->channel->name);
+		return ev;
+	} else {
+		return NULL;
 	}
-
-error:
-	return NULL;
 }
 
 /*
@@ -82,16 +137,15 @@ error:
  *
  * Return pointer to structure or NULL.
  */
-struct ltt_kernel_session *trace_kernel_create_session(char *path)
+struct ltt_kernel_session *trace_kernel_create_session(void)
 {
-	int ret;
-	struct ltt_kernel_session *lks;
+	struct ltt_kernel_session *lks = NULL;
 
 	/* Allocate a new ltt kernel session */
 	lks = zmalloc(sizeof(struct ltt_kernel_session));
 	if (lks == NULL) {
 		PERROR("create kernel session zmalloc");
-		goto error;
+		goto alloc_error;
 	}
 
 	/* Init data structure */
@@ -100,19 +154,19 @@ struct ltt_kernel_session *trace_kernel_create_session(char *path)
 	lks->channel_count = 0;
 	lks->stream_count_global = 0;
 	lks->metadata = NULL;
-	lks->consumer_fd = -1;
 	CDS_INIT_LIST_HEAD(&lks->channel_list.head);
 
-	/* Set session path */
-	ret = asprintf(&lks->trace_path, "%s/kernel", path);
-	if (ret < 0) {
-		PERROR("asprintf kernel traces path");
+	lks->consumer = consumer_create_output(CONSUMER_DST_LOCAL);
+	if (lks->consumer == NULL) {
 		goto error;
 	}
 
 	return lks;
 
 error:
+	free(lks);
+
+alloc_error:
 	return NULL;
 }
 
@@ -121,11 +175,13 @@ error:
  *
  * Return pointer to structure or NULL.
  */
-struct ltt_kernel_channel *trace_kernel_create_channel(struct lttng_channel *chan, char *path)
+struct ltt_kernel_channel *trace_kernel_create_channel(
+		struct lttng_channel *chan)
 {
-	int ret;
 	struct ltt_kernel_channel *lkc;
 
+	assert(chan);
+
 	lkc = zmalloc(sizeof(struct ltt_kernel_channel));
 	if (lkc == NULL) {
 		PERROR("ltt_kernel_channel zmalloc");
@@ -135,24 +191,29 @@ struct ltt_kernel_channel *trace_kernel_create_channel(struct lttng_channel *cha
 	lkc->channel = zmalloc(sizeof(struct lttng_channel));
 	if (lkc->channel == NULL) {
 		PERROR("lttng_channel zmalloc");
+		free(lkc);
 		goto error;
 	}
 	memcpy(lkc->channel, chan, sizeof(struct lttng_channel));
 
+	/*
+	 * If we receive an empty string for channel name, it means the
+	 * default channel name is requested.
+	 */
+	if (chan->name[0] == '\0') {
+		strncpy(lkc->channel->name, DEFAULT_CHANNEL_NAME,
+			sizeof(lkc->channel->name));
+	}
+	lkc->channel->name[LTTNG_KERNEL_SYM_NAME_LEN - 1] = '\0';
+
 	lkc->fd = -1;
 	lkc->stream_count = 0;
 	lkc->event_count = 0;
 	lkc->enabled = 1;
-	lkc->ctx = NULL;
 	/* Init linked list */
 	CDS_INIT_LIST_HEAD(&lkc->events_list.head);
 	CDS_INIT_LIST_HEAD(&lkc->stream_list.head);
-	/* Set default trace output path */
-	ret = asprintf(&lkc->pathname, "%s", path);
-	if (ret < 0) {
-		PERROR("asprintf kernel create channel");
-		goto error;
-	}
+	CDS_INIT_LIST_HEAD(&lkc->ctx_list);
 
 	return lkc;
 
@@ -160,16 +221,68 @@ error:
 	return NULL;
 }
 
+/*
+ * Allocate and init a kernel context object.
+ *
+ * Return the allocated object or NULL on error.
+ */
+struct ltt_kernel_context *trace_kernel_create_context(
+		struct lttng_kernel_context *ctx)
+{
+	struct ltt_kernel_context *kctx;
+
+	kctx = zmalloc(sizeof(*kctx));
+	if (!kctx) {
+		PERROR("zmalloc kernel context");
+		goto error;
+	}
+
+	if (ctx) {
+		memcpy(&kctx->ctx, ctx, sizeof(kctx->ctx));
+	}
+error:
+	return kctx;
+}
+
+/*
+ * Allocate and init a kernel context object from an existing kernel context
+ * object.
+ *
+ * Return the allocated object or NULL on error.
+ */
+struct ltt_kernel_context *trace_kernel_copy_context(
+		struct ltt_kernel_context *kctx)
+{
+	struct ltt_kernel_context *kctx_copy;
+
+	assert(kctx);
+	kctx_copy = zmalloc(sizeof(*kctx_copy));
+	if (!kctx_copy) {
+		PERROR("zmalloc ltt_kernel_context");
+		goto error;
+	}
+
+	memcpy(kctx_copy, kctx, sizeof(*kctx_copy));
+	memset(&kctx_copy->list, 0, sizeof(kctx_copy->list));
+
+error:
+	return kctx_copy;
+}
+
 /*
  * Allocate and initialize a kernel event. Set name and event type.
+ * We own filter_expression, and filter.
  *
  * Return pointer to structure or NULL.
  */
-struct ltt_kernel_event *trace_kernel_create_event(struct lttng_event *ev)
+struct ltt_kernel_event *trace_kernel_create_event(struct lttng_event *ev,
+		char *filter_expression, struct lttng_filter_bytecode *filter)
 {
 	struct ltt_kernel_event *lke;
 	struct lttng_kernel_event *attr;
 
+	assert(ev);
+
 	lke = zmalloc(sizeof(struct ltt_kernel_event));
 	attr = zmalloc(sizeof(struct lttng_kernel_event));
 	if (lke == NULL || attr == NULL) {
@@ -190,7 +303,6 @@ struct ltt_kernel_event *trace_kernel_create_event(struct lttng_event *ev)
 		attr->instrumentation = LTTNG_KERNEL_KRETPROBE;
 		attr->u.kretprobe.addr = ev->attr.probe.addr;
 		attr->u.kretprobe.offset = ev->attr.probe.offset;
-		attr->u.kretprobe.offset = ev->attr.probe.offset;
 		strncpy(attr->u.kretprobe.symbol_name,
 				ev->attr.probe.symbol_name, LTTNG_KERNEL_SYM_NAME_LEN);
 		attr->u.kretprobe.symbol_name[LTTNG_KERNEL_SYM_NAME_LEN - 1] = '\0';
@@ -223,11 +335,14 @@ struct ltt_kernel_event *trace_kernel_create_event(struct lttng_event *ev)
 	lke->fd = -1;
 	lke->event = attr;
 	lke->enabled = 1;
-	lke->ctx = NULL;
+	lke->filter_expression = filter_expression;
+	lke->filter = filter;
 
 	return lke;
 
 error:
+	free(filter_expression);
+	free(filter);
 	free(lke);
 	free(attr);
 	return NULL;
@@ -238,9 +353,8 @@ error:
  *
  * Return pointer to structure or NULL.
  */
-struct ltt_kernel_metadata *trace_kernel_create_metadata(char *path)
+struct ltt_kernel_metadata *trace_kernel_create_metadata(void)
 {
-	int ret;
 	struct ltt_kernel_metadata *lkm;
 	struct lttng_channel *chan;
 
@@ -253,21 +367,15 @@ struct ltt_kernel_metadata *trace_kernel_create_metadata(char *path)
 
 	/* Set default attributes */
 	chan->attr.overwrite = DEFAULT_CHANNEL_OVERWRITE;
-	chan->attr.subbuf_size = DEFAULT_METADATA_SUBBUF_SIZE;
+	chan->attr.subbuf_size = default_get_metadata_subbuf_size();
 	chan->attr.num_subbuf = DEFAULT_METADATA_SUBBUF_NUM;
-	chan->attr.switch_timer_interval = DEFAULT_CHANNEL_SWITCH_TIMER;
-	chan->attr.read_timer_interval = DEFAULT_CHANNEL_READ_TIMER;
+	chan->attr.switch_timer_interval = DEFAULT_KERNEL_CHANNEL_SWITCH_TIMER;
+	chan->attr.read_timer_interval = DEFAULT_KERNEL_CHANNEL_READ_TIMER;
 	chan->attr.output = DEFAULT_KERNEL_CHANNEL_OUTPUT;
 
 	/* Init metadata */
 	lkm->fd = -1;
 	lkm->conf = chan;
-	/* Set default metadata path */
-	ret = asprintf(&lkm->pathname, "%s/metadata", path);
-	if (ret < 0) {
-		PERROR("asprintf kernel metadata");
-		goto error;
-	}
 
 	return lkm;
 
@@ -283,20 +391,32 @@ error:
  *
  * Return pointer to structure or NULL.
  */
-struct ltt_kernel_stream *trace_kernel_create_stream(void)
+struct ltt_kernel_stream *trace_kernel_create_stream(const char *name,
+		unsigned int count)
 {
+	int ret;
 	struct ltt_kernel_stream *lks;
 
+	assert(name);
+
 	lks = zmalloc(sizeof(struct ltt_kernel_stream));
 	if (lks == NULL) {
 		PERROR("kernel stream zmalloc");
 		goto error;
 	}
 
+	/* Set name */
+	ret = snprintf(lks->name, sizeof(lks->name), "%s_%u", name, count);
+	if (ret < 0) {
+		PERROR("snprintf stream name");
+		goto error;
+	}
+	lks->name[sizeof(lks->name) - 1] = '\0';
+
 	/* Init stream */
 	lks->fd = -1;
-	lks->pathname = NULL;
 	lks->state = 0;
+	lks->cpu = count;
 
 	return lks;
 
@@ -309,11 +429,13 @@ error:
  */
 void trace_kernel_destroy_stream(struct ltt_kernel_stream *stream)
 {
-	int ret;
+	assert(stream);
 
 	DBG("[trace] Closing stream fd %d", stream->fd);
 	/* Close kernel fd */
 	if (stream->fd >= 0) {
+		int ret;
+
 		ret = close(stream->fd);
 		if (ret) {
 			PERROR("close");
@@ -322,7 +444,6 @@ void trace_kernel_destroy_stream(struct ltt_kernel_stream *stream)
 	/* Remove from stream list */
 	cds_list_del(&stream->list);
 
-	free(stream->pathname);
 	free(stream);
 }
 
@@ -331,9 +452,11 @@ void trace_kernel_destroy_stream(struct ltt_kernel_stream *stream)
  */
 void trace_kernel_destroy_event(struct ltt_kernel_event *event)
 {
-	int ret;
+	assert(event);
 
 	if (event->fd >= 0) {
+		int ret;
+
 		DBG("[trace] Closing event fd %d", event->fd);
 		/* Close kernel fd */
 		ret = close(event->fd);
@@ -347,11 +470,26 @@ void trace_kernel_destroy_event(struct ltt_kernel_event *event)
 	/* Remove from event list */
 	cds_list_del(&event->list);
 
+	free(event->filter_expression);
+	free(event->filter);
+
 	free(event->event);
-	free(event->ctx);
 	free(event);
 }
 
+/*
+ * Cleanup kernel context structure.
+ */
+void trace_kernel_destroy_context(struct ltt_kernel_context *ctx)
+{
+	assert(ctx);
+
+	if (ctx->in_list) {
+		cds_list_del(&ctx->list);
+	}
+	free(ctx);
+}
+
 /*
  * Cleanup kernel channel structure.
  */
@@ -359,8 +497,11 @@ void trace_kernel_destroy_channel(struct ltt_kernel_channel *channel)
 {
 	struct ltt_kernel_stream *stream, *stmp;
 	struct ltt_kernel_event *event, *etmp;
+	struct ltt_kernel_context *ctx, *ctmp;
 	int ret;
 
+	assert(channel);
+
 	DBG("[trace] Closing channel fd %d", channel->fd);
 	/* Close kernel fd */
 	if (channel->fd >= 0) {
@@ -380,12 +521,15 @@ void trace_kernel_destroy_channel(struct ltt_kernel_channel *channel)
 		trace_kernel_destroy_event(event);
 	}
 
+	/* For each context in the channel list */
+	cds_list_for_each_entry_safe(ctx, ctmp, &channel->ctx_list, list) {
+		trace_kernel_destroy_context(ctx);
+	}
+
 	/* Remove from channel list */
 	cds_list_del(&channel->list);
 
-	free(channel->pathname);
 	free(channel->channel);
-	free(channel->ctx);
 	free(channel);
 }
 
@@ -394,11 +538,13 @@ void trace_kernel_destroy_channel(struct ltt_kernel_channel *channel)
  */
 void trace_kernel_destroy_metadata(struct ltt_kernel_metadata *metadata)
 {
-	int ret;
+	assert(metadata);
 
 	DBG("[trace] Closing metadata fd %d", metadata->fd);
 	/* Close kernel fd */
 	if (metadata->fd >= 0) {
+		int ret;
+
 		ret = close(metadata->fd);
 		if (ret) {
 			PERROR("close");
@@ -406,18 +552,21 @@ void trace_kernel_destroy_metadata(struct ltt_kernel_metadata *metadata)
 	}
 
 	free(metadata->conf);
-	free(metadata->pathname);
 	free(metadata);
 }
 
 /*
  * Cleanup kernel session structure
+ *
+ * Should *NOT* be called with RCU read-side lock held.
  */
 void trace_kernel_destroy_session(struct ltt_kernel_session *session)
 {
 	struct ltt_kernel_channel *channel, *ctmp;
 	int ret;
 
+	assert(session);
+
 	DBG("[trace] Closing session fd %d", session->fd);
 	/* Close kernel fds */
 	if (session->fd >= 0) {
@@ -443,6 +592,8 @@ void trace_kernel_destroy_session(struct ltt_kernel_session *session)
 		trace_kernel_destroy_channel(channel);
 	}
 
-	free(session->trace_path);
+	/* Wipe consumer output object */
+	consumer_output_put(session->consumer);
+
 	free(session);
 }
