@@ -168,6 +168,7 @@ int cmd_destroy(int argc, const char **argv)
 	int ret = CMD_SUCCESS , i, command_ret = CMD_SUCCESS, success = 1;
 	static poptContext pc;
 	char *session_name = NULL;
+	const char *leftover = NULL;
 
 	struct lttng_session *sessions;
 	int count;
@@ -226,7 +227,8 @@ int cmd_destroy(int argc, const char **argv)
 	/* Recuperate all sessions for further operation */
 	count = lttng_list_sessions(&sessions);
 	if (count < 0) {
-		command_ret = count;
+		ERR("%s", lttng_strerror(count));
+		command_ret = CMD_ERROR;
 		success = 0;
 		goto mi_closing;
 	}
@@ -273,6 +275,14 @@ int cmd_destroy(int argc, const char **argv)
 		}
 	}
 
+	leftover = poptGetArg(pc);
+	if (leftover) {
+		ERR("Unknown argument: %s", leftover);
+		ret = CMD_ERROR;
+		success = 0;
+		goto mi_closing;
+	}
+
 mi_closing:
 	/* Mi closing */
 	if (lttng_opt_mi) {
