@@ -15,7 +15,7 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-#define _GNU_SOURCE
+#define _LGPL_SOURCE
 #include <popt.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -23,6 +23,9 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <unistd.h>
+#include <assert.h>
+
+#include <common/mi-lttng.h>
 
 #include "../command.h"
 
@@ -30,11 +33,6 @@ static char *opt_channels;
 static int opt_kernel;
 static char *opt_session_name;
 static int opt_userspace;
-#if 0
-/* Not implemented yet */
-static char *opt_cmd_name;
-static pid_t opt_pid;
-#endif
 
 enum {
 	OPT_HELP = 1,
@@ -43,43 +41,58 @@ enum {
 };
 
 static struct lttng_handle *handle;
+static struct mi_writer *writer;
 
 static struct poptOption long_options[] = {
 	/* longName, shortName, argInfo, argPtr, value, descrip, argDesc */
 	{"help",           'h', POPT_ARG_NONE, 0, OPT_HELP, 0, 0},
 	{"session",        's', POPT_ARG_STRING, &opt_session_name, 0, 0, 0},
 	{"kernel",         'k', POPT_ARG_VAL, &opt_kernel, 1, 0, 0},
-#if 0
-	/* Not implemented yet */
-	{"userspace",      'u', POPT_ARG_STRING | POPT_ARGFLAG_OPTIONAL, &opt_cmd_name, OPT_USERSPACE, 0, 0},
-	{"pid",            'p', POPT_ARG_INT, &opt_pid, 0, 0, 0},
-#else
 	{"userspace",      'u', POPT_ARG_NONE, 0, OPT_USERSPACE, 0, 0},
-#endif
 	{"list-options", 0, POPT_ARG_NONE, NULL, OPT_LIST_OPTIONS, NULL, NULL},
 	{0, 0, 0, 0, 0, 0, 0}
 };
 
-/*
- * usage
- */
-static void usage(FILE *ofp)
+static int mi_partial_channel_print(char *channel_name, unsigned int enabled,
+		int success)
 {
-	fprintf(ofp, "usage: lttng disable-channel NAME[,NAME2,...] [options]\n");
-	fprintf(ofp, "\n");
-	fprintf(ofp, "  -h, --help               Show this help\n");
-	fprintf(ofp, "      --list-options       Simple listing of options\n");
-	fprintf(ofp, "  -s, --session            Apply to session name\n");
-	fprintf(ofp, "  -k, --kernel             Apply to the kernel tracer\n");
-#if 0
-	fprintf(ofp, "  -u, --userspace [CMD]    Apply to the user-space tracer\n");
-	fprintf(ofp, "                           If no CMD, the domain used is UST global\n");
-	fprintf(ofp, "                           or else the domain is UST EXEC_NAME\n");
-	fprintf(ofp, "  -p, --pid PID            If -u, apply to specific PID (domain: UST PID)\n");
-#else
-	fprintf(ofp, "  -u, --userspace          Apply to the user-space tracer\n");
-#endif
-	fprintf(ofp, "\n");
+	int ret;
+
+	assert(writer);
+	assert(channel_name);
+
+	/* Open channel element */
+	ret = mi_lttng_writer_open_element(writer, config_element_channel);
+	if (ret) {
+		goto end;
+	}
+
+	/* Name */
+	ret = mi_lttng_writer_write_element_string(writer, config_element_name,
+			channel_name);
+	if (ret) {
+		goto end;
+	}
+
+	/* Enabled ? */
+	ret = mi_lttng_writer_write_element_bool(writer, config_element_enabled,
+			enabled);
+	if (ret) {
+		goto end;
+	}
+
+	/* Success ? */
+	ret = mi_lttng_writer_write_element_bool(writer,
+			mi_lttng_element_success, success);
+	if (ret) {
+		goto end;
+	}
+
+	/* Closing channel element */
+	ret = mi_lttng_writer_close_element(writer);
+
+end:
+	return ret;
 }
 
 /*
@@ -87,7 +100,10 @@ static void usage(FILE *ofp)
  */
 static int disable_channels(char *session_name)
 {
-	int ret = CMD_SUCCESS, warn = 0;
+	int ret = CMD_SUCCESS, warn = 0, success;
+
+	/* Normal case for disable channed is enabled = 0 */
+	unsigned int enabled = 0;
 	char *channel_name;
 	struct lttng_domain dom;
 
@@ -99,9 +115,8 @@ static int disable_channels(char *session_name)
 	} else if (opt_userspace) {
 		dom.type = LTTNG_DOMAIN_UST;
 	} else {
-		ERR("Please specify a tracer (-k/--kernel or -u/--userspace)");
-		ret = CMD_ERROR;
-		goto error;
+		/* Checked by the caller. */
+		assert(0);
 	}
 
 	handle = lttng_create_handle(session_name, &dom);
@@ -110,6 +125,17 @@ static int disable_channels(char *session_name)
 		goto error;
 	}
 
+	/* Prepare MI */
+	if (lttng_opt_mi) {
+		/* open a channels element */
+		ret = mi_lttng_writer_open_element(writer, config_element_channels);
+		if (ret) {
+			ret = CMD_ERROR;
+			goto error;
+		}
+
+	}
+
 	/* Strip channel list */
 	channel_name = strtok(opt_channels, ",");
 	while (channel_name != NULL) {
@@ -120,9 +146,31 @@ static int disable_channels(char *session_name)
 			ERR("Channel %s: %s (session %s)", channel_name,
 					lttng_strerror(ret), session_name);
 			warn = 1;
+
+			/*
+			 * Mi:
+			 * We assume that if an error occurred the channel is still active.
+			 * This might not be the case but is a good assumption.
+			 * The client should look at the stderr stream
+			 * for more informations.
+			 */
+			enabled = 1;
+			success = 0;
+
 		} else {
 			MSG("%s channel %s disabled for session %s",
-					opt_kernel ? "Kernel" : "UST", channel_name, session_name);
+					get_domain_str(dom.type), channel_name, session_name);
+			enabled = 0;
+			success = 1;
+		}
+
+		/* Print the channel */
+		if (lttng_opt_mi) {
+			ret = mi_partial_channel_print(channel_name, enabled, success);
+			if (ret) {
+				ret = CMD_ERROR;
+				goto error;
+			}
 		}
 
 		/* Next channel */
@@ -131,8 +179,19 @@ static int disable_channels(char *session_name)
 
 	ret = CMD_SUCCESS;
 
+	/* Close Mi */
+	if (lttng_opt_mi) {
+		/* Close channels element */
+		ret = mi_lttng_writer_close_element(writer);
+		if (ret) {
+			ret = CMD_ERROR;
+			goto error;
+		}
+	}
+
 error:
-	if (warn) {
+	/* Bypass the warning if a more important error happened */
+	if (!ret && warn) {
 		ret = CMD_WARNING;
 	}
 
@@ -148,9 +207,10 @@ error:
  */
 int cmd_disable_channels(int argc, const char **argv)
 {
-	int opt, ret = CMD_SUCCESS;
+	int opt, ret = CMD_SUCCESS, command_ret = CMD_SUCCESS, success = 1;
 	static poptContext pc;
 	char *session_name = NULL;
+	const char *leftover = NULL;
 
 	pc = poptGetContext(NULL, argc, argv, long_options, 0);
 	poptReadDefaultConfig(pc, 0);
@@ -158,7 +218,7 @@ int cmd_disable_channels(int argc, const char **argv)
 	while ((opt = poptGetNextOpt(pc)) != -1) {
 		switch (opt) {
 		case OPT_HELP:
-			usage(stdout);
+			SHOW_HELP();
 			goto end;
 		case OPT_USERSPACE:
 			opt_userspace = 1;
@@ -167,16 +227,27 @@ int cmd_disable_channels(int argc, const char **argv)
 			list_cmd_options(stdout, long_options);
 			goto end;
 		default:
-			usage(stderr);
 			ret = CMD_UNDEFINED;
 			goto end;
 		}
 	}
 
+	ret = print_missing_or_multiple_domains(opt_kernel + opt_userspace);
+	if (ret) {
+		ret = CMD_ERROR;
+		goto end;
+	}
+
 	opt_channels = (char*) poptGetArg(pc);
 	if (opt_channels == NULL) {
 		ERR("Missing channel name(s).\n");
-		usage(stderr);
+		ret = CMD_ERROR;
+		goto end;
+	}
+
+	leftover = poptGetArg(pc);
+	if (leftover) {
+		ERR("Unknown argument: %s", leftover);
 		ret = CMD_ERROR;
 		goto end;
 	}
@@ -191,12 +262,75 @@ int cmd_disable_channels(int argc, const char **argv)
 		session_name = opt_session_name;
 	}
 
-	ret = disable_channels(session_name);
+	/* Mi check */
+	if (lttng_opt_mi) {
+		writer = mi_lttng_writer_create(fileno(stdout), lttng_opt_mi);
+		if (!writer) {
+			ret = -LTTNG_ERR_NOMEM;
+			goto end;
+		}
+
+		/* Open command element */
+		ret = mi_lttng_writer_command_open(writer,
+				mi_lttng_element_command_disable_channel);
+		if (ret) {
+			ret = CMD_ERROR;
+			goto end;
+		}
+
+		/* Open output element */
+		ret = mi_lttng_writer_open_element(writer,
+				mi_lttng_element_command_output);
+		if (ret) {
+			ret = CMD_ERROR;
+			goto end;
+		}
+	}
+
+	command_ret = disable_channels(session_name);
+	if (command_ret) {
+		success = 0;
+	}
+
+	/* Mi closing */
+	if (lttng_opt_mi) {
+		/* Close  output element */
+		ret = mi_lttng_writer_close_element(writer);
+		if (ret) {
+			ret = CMD_ERROR;
+			goto end;
+		}
+
+		/* Success ? */
+		ret = mi_lttng_writer_write_element_bool(writer,
+				mi_lttng_element_success, success);
+		if (ret) {
+			ret = CMD_ERROR;
+			goto end;
+		}
+
+		/* Command element close */
+		ret = mi_lttng_writer_command_close(writer);
+		if (ret) {
+			ret = CMD_ERROR;
+			goto end;
+		}
+	}
 
 end:
+	/* Mi clean-up */
+	if (writer && mi_lttng_writer_destroy(writer)) {
+		/* Preserve original error code */
+		ret = ret ? ret : LTTNG_ERR_MI_IO_FAIL;
+	}
+
 	if (!opt_session_name && session_name) {
 		free(session_name);
 	}
+
+	/* Overwrite ret if an error occurred in disable_channels */
+	ret = command_ret ? command_ret : ret;
+
 	poptFreeContext(pc);
 	return ret;
 }
