@@ -296,6 +296,7 @@ static int view_trace(void)
 	int ret;
 	char *session_name, *trace_path = NULL;
 	struct lttng_session *sessions = NULL;
+	bool free_trace_path = false;
 
 	/*
 	 * Safety net. If lttng is suid at some point for *any* useless reasons,
@@ -373,6 +374,7 @@ static int view_trace(void)
 				ret = CMD_ERROR;
 				goto free_sessions;
 			}
+			free_trace_path = true;
 		} else {
 			/* Get file system session path. */
 			trace_path = sessions[i].path;
@@ -390,7 +392,7 @@ static int view_trace(void)
 	}
 
 free_sessions:
-	if (session_live_mode) {
+	if (session_live_mode && free_trace_path) {
 		free(trace_path);
 	}
 	free(sessions);
@@ -409,6 +411,7 @@ int cmd_view(int argc, const char **argv)
 {
 	int opt, ret = CMD_SUCCESS;
 	static poptContext pc;
+	const char *leftover = NULL;
 
 	pc = poptGetContext(NULL, argc, argv, long_options, 0);
 	poptReadDefaultConfig(pc, 0);
@@ -433,6 +436,13 @@ int cmd_view(int argc, const char **argv)
 
 	opt_session_name = (char*) poptGetArg(pc);
 
+	leftover = poptGetArg(pc);
+	if (leftover) {
+		ERR("Unknown argument: %s", leftover);
+		ret = CMD_ERROR;
+		goto end;
+	}
+
 	ret = view_trace();
 
 end:
